@@ -1,5 +1,6 @@
-from ...ocr.gsm_ocr_config import set_dpi_awareness, get_scene_ocr_config_path, OCRConfig, get_scene_ocr_config
-from ...util.electron_config import *
+from GameSentenceMiner.ocr.gsm_ocr_config import set_dpi_awareness, get_scene_ocr_config
+from GameSentenceMiner.util.electron_config import *  # noqa: F403
+from GameSentenceMiner.util.gsm_utils import do_text_replacements, OCR_REPLACEMENTS_FILE
 
 try:
     import win32gui
@@ -28,7 +29,6 @@ import signal
 import threading
 from pathlib import Path
 import queue
-import io
 import re
 import logging
 import inspect
@@ -39,23 +39,23 @@ import mss
 import asyncio
 import websockets
 import socketserver
-import queue
+import cv2
+import numpy as np
 
+from collections import deque
 from datetime import datetime, timedelta
-from PIL import Image, ImageDraw, UnidentifiedImageError
+from PIL import Image, ImageDraw
 from loguru import logger
 from desktop_notifier import DesktopNotifierSync
 import psutil
 
-import inspect
-from .ocr import *
-try:
-    from .secret import *
-except ImportError:
-    pass
+from .ocr import *  # noqa: F403
 from .config import Config
 from .screen_coordinate_picker import get_screen_selection
-from GameSentenceMiner.util.configuration import get_temporary_directory, get_config
+from GameSentenceMiner.util.configuration import get_temporary_directory
+
+from skimage.metrics import structural_similarity as ssim
+from typing import Union
 
 config = None
 last_image = None
@@ -338,6 +338,7 @@ class TextFiltering:
         self.thai_regex = re.compile(r'[\u0E00-\u0E7F]')
         self.latin_extended_regex = re.compile(
             r'[a-zA-Z\u00C0-\u00FF\u0100-\u017F\u0180-\u024F\u0250-\u02AF\u1D00-\u1D7F\u1D80-\u1DBF\u1E00-\u1EFF\u2C60-\u2C7F\uA720-\uA7FF\uAB30-\uAB6F]')
+        self.last_few_results = {}
         try:
             from transformers import pipeline, AutoTokenizer
             import torch
@@ -362,7 +363,7 @@ class TextFiltering:
             import langid
             self.classify = langid.classify
 
-    def __call__(self, text, last_result):
+    def __call__(self, text, last_result, engine=None, is_second_ocr=False):
         lang = get_ocr_language()
         if self.initial_lang != lang:
             from pysbd import Segmenter
@@ -403,11 +404,24 @@ class TextFiltering:
 
         try:
             if isinstance(last_result, list):
-                last_text = last_result
+                last_text = last_result.copy()
             elif last_result and last_result[1] == engine_index:
                 last_text = last_result[0]
             else:
                 last_text = []
+            
+            if engine and not is_second_ocr:
+                if self.last_few_results and self.last_few_results.get(engine):
+                    for sublist in self.last_few_results.get(engine, []):
+                        if sublist:
+                            for item in sublist:
+                                if item and item not in last_text:
+                                    last_text.append(item)
+                    self.last_few_results[engine].append(orig_text_filtered)
+                else:
+                    self.last_few_results[engine] = deque(maxlen=3)
+                    self.last_few_results[engine].append(orig_text_filtered)
+
         except Exception as e:
             logger.error(f"Error processing last_result {last_result}: {e}")
             last_text = []
@@ -799,8 +813,6 @@ class ScreenshotThread(threading.Thread):
             self.windows_window_tracker_instance.join()
 
 
-import cv2
-import numpy as np
     
 def apply_adaptive_threshold_filter(img):
     img = cv2.cvtColor(np.array(img), cv2.COLOR_RGB2BGR)
@@ -865,11 +877,6 @@ def are_images_identical(img1, img2, img2_np=None):
     return (img1_np.shape == img2_np.shape) and np.array_equal(img1_np, img2_np)
 
 
-import cv2
-import numpy as np
-from skimage.metrics import structural_similarity as ssim
-from typing import Union
-
 ImageType = Union[np.ndarray, Image.Image]
 
 def _prepare_image(image: ImageType) -> np.ndarray:
@@ -989,7 +996,7 @@ def quick_text_detection(pil_image, threshold_ratio=0.01):
 
 # Use OBS for Screenshot Source (i.e. Linux)
 class OBSScreenshotThread(threading.Thread):
-    def __init__(self, ocr_config, screen_capture_on_combo, width=1280, height=720, interval=1):
+    def __init__(self, ocr_config, screen_capture_on_combo, width=1280, height=720, interval=1, is_manual_ocr=False):
         super().__init__(daemon=True)
         self.ocr_config = ocr_config
         self.interval = interval
@@ -1000,6 +1007,7 @@ class OBSScreenshotThread(threading.Thread):
         self.width = width
         self.height = height
         self.use_periodic_queue = not screen_capture_on_combo
+        self.is_manual_ocr = is_manual_ocr
 
     def write_result(self, result):
         if self.use_periodic_queue:
@@ -1011,62 +1019,26 @@ class OBSScreenshotThread(threading.Thread):
     def connect_obs(self):
         import GameSentenceMiner.obs as obs
         obs.connect_to_obs_sync(check_output=False)
-
-    def scale_down_width_height(self, width, height):
-        if width == 0 or height == 0:
-            return self.width, self.height
-        # return width, height
-        aspect_ratio = width / height
-        logger.info(
-            f"Scaling down OBS source dimensions: {width}x{height} (Aspect Ratio: {aspect_ratio})")
-        if aspect_ratio > 2.66:
-            # Ultra-wide (32:9) - use 1920x540
-            logger.info("Using ultra-wide aspect ratio scaling (32:9).")
-            return 1920, 540
-        elif aspect_ratio > 2.33:
-            # 21:9 - use 1920x800
-            logger.info("Using ultra-wide aspect ratio scaling (21:9).")
-            return 1920, 800
-        elif aspect_ratio > 1.77:
-            # 16:9 - use 1280x720
-            logger.info("Using standard aspect ratio scaling (16:9).")
-            return 1280, 720
-        elif aspect_ratio > 1.6:
-            # 16:10 - use 1280x800
-            logger.info("Using standard aspect ratio scaling (16:10).")
-            return 1280, 800
-        elif aspect_ratio > 1.33:
-            # 4:3 - use 960x720
-            logger.info("Using standard aspect ratio scaling (4:3).")
-            return 960, 720
-        elif aspect_ratio > 1.25:
-            # 5:4 - use 900x720
-            logger.info("Using standard aspect ratio scaling (5:4).")
-            return 900, 720
-        elif aspect_ratio > 1.5:
-            # 3:2 - use 1080x720
-            logger.info("Using standard aspect ratio scaling (3:2).")
-            return 1080, 720
-        else:
-            # Default fallback - use original resolution
-            logger.info(
-                "Using default aspect ratio scaling (original resolution).")
-            return width, height
         
     def init_config(self, source=None, scene=None):
         import GameSentenceMiner.obs as obs
         obs.update_current_game()
         self.current_source = source if source else obs.get_active_source()
-        logger.info(f"Current OBS source: {self.current_source}")
+        logger.debug(f"Current OBS source: {self.current_source}")
         self.source_width = self.current_source.get(
             "sceneItemTransform").get("sourceWidth") or self.width
         self.source_height = self.current_source.get(
             "sceneItemTransform").get("sourceHeight") or self.height
-        if self.source_width and self.source_height:
-            self.width, self.height = self.scale_down_width_height(
+        if self.source_width and self.source_height and not self.is_manual_ocr and not get_ocr_two_pass_ocr():
+            self.width, self.height = scale_down_width_height(
                 self.source_width, self.source_height)
             logger.info(
-                f"Using OBS source dimensions: {self.width}x{self.height}")
+                f"Using OBS source dimensions: {self.source_width}x{self.source_height}")
+        else:
+            self.width = self.source_width or 1280
+            self.height = self.source_height or 720
+            logger.info(
+                f"Using source dimensions: {self.width}x{self.height}")
         self.current_source_name = self.current_source.get(
             "sourceName") or None
         self.current_scene = scene if scene else obs.get_current_game()
@@ -1113,7 +1085,7 @@ class OBSScreenshotThread(threading.Thread):
                     self.write_result(1)
                     continue
                 img = obs.get_screenshot_PIL(source_name=self.current_source_name,
-                                             width=self.width, height=self.height, img_format='jpg', compression=80)
+                                             width=self.width, height=self.height, img_format='jpg', compression=100)
                 
                 img = apply_ocr_config_to_image(img, self.ocr_config)
 
@@ -1128,6 +1100,39 @@ class OBSScreenshotThread(threading.Thread):
                     f"An unexpected error occurred during OBS Capture : {e}", exc_info=True)
                 time.sleep(.5)
                 continue
+            
+def scale_down_width_height(width, height):
+        if width == 0 or height == 0:
+            return width, height
+        # return width, height
+        aspect_ratio = width / height
+        logger.info(
+            f"Scaling down OBS source dimensions: {width}x{height} (Aspect Ratio: {aspect_ratio})")
+        if aspect_ratio > 2.66:
+            logger.info("Using ultra-wide aspect ratio scaling (32:9).")
+            return 1920, 540
+        elif aspect_ratio > 2.33:
+            logger.info("Using ultra-wide aspect ratio scaling (21:9).")
+            return 1920, 800
+        elif aspect_ratio > 1.77:
+            logger.info("Using standard aspect ratio scaling (16:9).")
+            return 1280, 720
+        elif aspect_ratio > 1.6:
+            logger.info("Using standard aspect ratio scaling (16:10).")
+            return 1280, 800
+        elif aspect_ratio > 1.33:
+            logger.info("Using standard aspect ratio scaling (4:3).")
+            return 960, 720
+        elif aspect_ratio > 1.25:
+            logger.info("Using standard aspect ratio scaling (5:4).")
+            return 900, 720
+        elif aspect_ratio > 1.5:
+            logger.info("Using standard aspect ratio scaling (3:2).")
+            return 1080, 720
+        else:
+            logger.info(
+                "Using default aspect ratio scaling (original resolution).")
+            return width, height
 
 
 def apply_ocr_config_to_image(img, ocr_config, is_secondary=False):
@@ -1319,10 +1324,16 @@ def on_screenshot_combo():
 def on_window_minimized(minimized):
     global screencapture_window_visible
     screencapture_window_visible = not minimized
+    
+
+def do_configured_ocr_replacements(text: str) -> str:
+    return do_text_replacements(text, OCR_REPLACEMENTS_FILE)
 
 
-def process_and_write_results(img_or_path, write_to=None, last_result=None, filtering=None, notify=None, engine=None, ocr_start_time=None, furigana_filter_sensitivity=0):
+def process_and_write_results(img_or_path, write_to=None, last_result=None, filtering: TextFiltering = None, notify=None, engine=None, ocr_start_time=None, furigana_filter_sensitivity=0):
     global engine_index
+    # TODO Replace this at a later date
+    is_second_ocr = bool(engine)
     if auto_pause_handler:
         auto_pause_handler.stop()
     if engine:
@@ -1332,9 +1343,10 @@ def process_and_write_results(img_or_path, write_to=None, last_result=None, filt
                 break
     else:
         engine_instance = engine_instances[engine_index]
+        engine = engine_instance.name
 
     engine_color = config.get_general('engine_color')
-
+    
     start_time = time.time()
     result = engine_instance(img_or_path, furigana_filter_sensitivity)
     res, text, crop_coords = (*result, None)[:3]
@@ -1364,14 +1376,21 @@ def process_and_write_results(img_or_path, write_to=None, last_result=None, filt
     # print(engine_index)
 
     if res:
+        text = do_configured_ocr_replacements(text)
         if filtering:
-            text, orig_text = filtering(text, last_result)
+            text, orig_text = filtering(text, last_result, engine=engine, is_second_ocr=is_second_ocr)
         if get_ocr_language() == "ja" or get_ocr_language() == "zh":
             text = post_process(text, keep_blank_lines=get_ocr_keep_newline())
-        logger.opt(ansi=True).info(
-            f'Text recognized in {end_time - start_time:0.03f}s using <{engine_color}>{engine_instance.readable_name}</{engine_color}>: {text}')
         if notify and config.get_general('notifications'):
             notifier.send(title='owocr', message='Text recognized: ' + text)
+            
+        if text and write_to is not None:
+            if check_text_is_all_menu(text, crop_coords):
+                logger.opt(ansi=True).info('Text is identified as all menu items, skipping further processing.')
+                return orig_text, ''
+            
+        logger.opt(ansi=True).info(
+    f'Text recognized in {end_time - start_time:0.03f}s using <{engine_color}>{engine_instance.readable_name}</{engine_color}>: {text}')
 
         if write_to == 'websocket':
             websocket_server_thread.send_text(text)
@@ -1379,7 +1398,7 @@ def process_and_write_results(img_or_path, write_to=None, last_result=None, filt
             pyperclipfix.copy(text)
         elif write_to == "callback":
             txt_callback(text, orig_text, ocr_start_time,
-                         img_or_path, bool(engine), filtering, crop_coords)
+                         img_or_path, is_second_ocr, filtering, crop_coords)
         elif write_to:
             with Path(write_to).open('a', encoding='utf-8') as f:
                 f.write(text + '\n')
@@ -1395,6 +1414,89 @@ def process_and_write_results(img_or_path, write_to=None, last_result=None, filt
 
     return orig_text, text
 
+def check_text_is_all_menu(text: str, crop_coords: tuple) -> bool:
+    """
+    Checks if the recognized text consists entirely of menu items.
+    This function checks if the detected text area falls entirely within secondary rectangles (menu areas).
+
+    :param text: The recognized text from OCR.
+    :param crop_coords: Tuple containing (x, y, x2, y2) of the detected text area relative to the cropped image.
+    :return: True if the text is all menu items (within secondary rectangles), False otherwise.
+    """
+    if not text or not crop_coords:
+        return False
+
+    original_width = obs_screenshot_thread.width
+    original_height = obs_screenshot_thread.height
+    crop_x, crop_y, crop_x2, crop_y2 = crop_coords
+
+    ocr_config = get_scene_ocr_config()
+    
+    if not any(rect.is_secondary for rect in ocr_config.rectangles):
+        return False
+
+    ocr_config.scale_to_custom_size(original_width, original_height)
+    if not ocr_config or not ocr_config.rectangles:
+        return False
+
+    primary_rectangles = [rect for rect in ocr_config.rectangles if not rect.is_excluded and not rect.is_secondary]
+    menu_rectangles = [rect for rect in ocr_config.rectangles if rect.is_secondary and not rect.is_excluded]
+
+    if not menu_rectangles:
+        return False
+
+    if not primary_rectangles:
+        if crop_x < 0 or crop_y < 0 or crop_x2 > original_width or crop_y2 > original_height:
+            return False
+        for menu_rect in menu_rectangles:
+            rect_left, rect_top, rect_width, rect_height = menu_rect.coordinates
+            rect_right = rect_left + rect_width
+            rect_bottom = rect_top + rect_height
+            if (crop_x >= rect_left and crop_y >= rect_top and
+                crop_x2 <= rect_right and crop_y2 <= rect_bottom):
+                return True
+        return False
+
+    primary_rectangles.sort(key=lambda r: r.coordinates[1])
+
+    if len(primary_rectangles) == 1:
+        primary_rect = primary_rectangles[0]
+        primary_left, primary_top, primary_width, primary_height = primary_rect.coordinates
+        original_x = crop_x + primary_left
+        original_y = crop_y + primary_top
+        original_x2 = crop_x2 + primary_left
+        original_y2 = crop_y2 + primary_top
+    else:
+        current_y_offset = 0
+        original_x = None
+        original_y = None
+        original_x2 = None
+        original_y2 = None
+        for i, primary_rect in enumerate(primary_rectangles):
+            primary_left, primary_top, primary_width, primary_height = primary_rect.coordinates
+            section_height = primary_height
+            if crop_y >= current_y_offset and crop_y < current_y_offset + section_height:
+                original_x = crop_x + primary_left
+                original_y = (crop_y - current_y_offset) + primary_top
+                original_x2 = crop_x2 + primary_left
+                original_y2 = crop_y2 + primary_top
+                break
+            current_y_offset += section_height + 50
+        if original_x is None or original_y is None:
+            return False
+
+    if original_x < 0 or original_y < 0 or original_x > original_width or original_y > original_height:
+        return False
+
+    for menu_rect in menu_rectangles:
+        rect_left, rect_top, rect_width, rect_height = menu_rect.coordinates
+        rect_right = rect_left + rect_width
+        rect_bottom = rect_top + rect_height
+        if (original_x >= rect_left and original_y >= rect_top and
+            original_x2 <= rect_right and original_y2 <= rect_bottom):
+            return True
+
+    return False
 
 def get_path_key(path):
     return path, path.lstat().st_mtime
@@ -1632,7 +1734,7 @@ def run(read_from=None,
         last_result = ([], engine_index)
         screenshot_event = threading.Event()
         obs_screenshot_thread = OBSScreenshotThread(
-            gsm_ocr_config, screen_capture_on_combo, interval=screen_capture_delay_secs)
+            gsm_ocr_config, screen_capture_on_combo, interval=screen_capture_delay_secs, is_manual_ocr=bool(screen_capture_on_combo))
         obs_screenshot_thread.start()
         filtering = TextFiltering()
         read_from_readable.append('obs')
@@ -1785,7 +1887,7 @@ def run(read_from=None,
                     continue
 
                 res, text = process_and_write_results(img, write_to, last_result, filtering, notify,
-                                                   ocr_start_time=ocr_start_time, furigana_filter_sensitivity=get_ocr_furigana_filter_sensitivity())
+                                                   ocr_start_time=ocr_start_time, furigana_filter_sensitivity=None if get_ocr_two_pass_ocr() else get_furigana_filter_sensitivity())
                 if not text and not previous_text and time.time() - last_result_time > 10:
                     sleep_time_to_add += .005
                     logger.info(f"No text detected again, sleeping.")
