@@ -436,6 +436,7 @@ class Anki:
     word_field: str = 'Expression'
     previous_sentence_field: str = ''
     previous_image_field: str = ''
+    video_field: str = ''
     # Initialize to None and set it in __post_init__
     custom_tags: List[str] = None
     tags_to_check: List[str] = None
@@ -475,6 +476,7 @@ class Screenshot:
     custom_ffmpeg_settings: str = ''
     custom_ffmpeg_option_selected: str = ''
     screenshot_hotkey_updates_anki: bool = False
+    animated: bool = False
     seconds_after_line: float = 1.0
     use_beginning_of_line_as_screenshot: bool = True
     use_new_screenshot_logic: bool = False
@@ -502,7 +504,7 @@ class Audio:
     beginning_offset: float = -0.5
     end_offset: float = 0.5
     pre_vad_end_offset: float = 0.0
-    ffmpeg_reencode_options: str = '-c:a {encoder} -f {format} -af \"afade=t=in:d=0.10\"' if is_windows() else ''
+    ffmpeg_reencode_options: str = '-c:a {encoder} -f {format} -af \"afade=t=in:d=0.005\"' if is_windows() else ''
     ffmpeg_reencode_options_to_use: str = ''
     external_tool: str = ""
     anki_media_collection: str = ""
@@ -557,6 +559,7 @@ class VAD:
     add_audio_on_no_results: bool = False
     cut_and_splice_segments: bool = False
     splice_padding: float = 0.1
+    use_cpu_for_inference: bool = False
 
     def is_silero(self):
         return self.selected_vad_model == SILERO or self.backup_vad_model == SILERO
@@ -1147,10 +1150,12 @@ class AnkiUpdateResult:
     prev_screenshot_in_anki: str = ''
     sentence_in_anki: str = ''
     multi_line: bool = False
+    video_in_anki: str = ''
+    word_path: str = ''
 
     @staticmethod
     def failure():
-        return AnkiUpdateResult(success=False, audio_in_anki='', screenshot_in_anki='', prev_screenshot_in_anki='', sentence_in_anki='', multi_line=False)
+        return AnkiUpdateResult(success=False, audio_in_anki='', screenshot_in_anki='', prev_screenshot_in_anki='', sentence_in_anki='', multi_line=False, video_in_anki='', word_path='')
 
 
 @dataclass_json
