@@ -139,7 +139,7 @@ class VADProcessor(ABC):
             self.extract_audio_and_combine_segments(input_audio, voice_activity, output_audio, padding=get_config().vad.splice_padding)
         else:
             ffmpeg.trim_audio(input_audio, start_time + get_config().vad.beginning_offset, end_time + get_config().audio.end_offset, output_audio, trim_beginning=get_config().vad.trim_beginning, fade_in_duration=0.05, fade_out_duration=0)
-        return VADResult(True, start_time + get_config().vad.beginning_offset, end_time + get_config().audio.end_offset, self.vad_system_name, voice_activity, output_audio)
+        return VADResult(True, max(0, start_time + get_config().vad.beginning_offset), max(0, end_time + get_config().audio.end_offset), self.vad_system_name, voice_activity, output_audio)
 
 class SileroVADProcessor(VADProcessor):
     def __init__(self):
@@ -150,9 +150,9 @@ class SileroVADProcessor(VADProcessor):
 
     def _detect_voice_activity(self, input_audio):
         from silero_vad import read_audio, get_speech_timestamps
-        # temp_wav = tempfile.NamedTemporaryFile(dir=configuration.get_temporary_directory(), suffix='.wav').name
-        # ffmpeg.convert_audio_to_wav(input_audio, temp_wav)
-        wav = read_audio(input_audio)
+        temp_wav = tempfile.NamedTemporaryFile(dir=configuration.get_temporary_directory(), suffix='.wav').name
+        ffmpeg.convert_audio_to_wav(input_audio, temp_wav)
+        wav = read_audio(temp_wav)
         speech_timestamps = get_speech_timestamps(wav, self.vad_model, return_seconds=True, threshold=0.2)
         logger.debug(speech_timestamps)
         return speech_timestamps
@@ -165,27 +165,31 @@ class WhisperVADProcessor(VADProcessor):
 
     def load_whisper_model(self):
         import stable_whisper as whisper
+        import torch
         if not self.vad_model:
+            self.device = "cpu" if get_config().vad.use_cpu_for_inference else "cuda" if torch.cuda.is_available() else "cpu"
             with warnings.catch_warnings():
-                self.vad_model = whisper.load_model(get_config().vad.whisper_model)
+                warnings.simplefilter("ignore")
+                self.vad_model = whisper.load_faster_whisper(get_config().vad.whisper_model, device=self.device)
             logger.info(f"Whisper model '{get_config().vad.whisper_model}' loaded.")
         return self.vad_model
 
     def _detect_voice_activity(self, input_audio):
         from stable_whisper import WhisperResult
-        # Convert the audio to 16kHz mono WAV
-        # temp_wav = tempfile.NamedTemporaryFile(dir=configuration.get_temporary_directory(), suffix='.wav').name
-        # ffmpeg.convert_audio_to_wav(input_audio, temp_wav)
+        # Convert the audio to 16kHz mono WAV, evidence https://discord.com/channels/1286409772383342664/1286518821913362445/1407017127529152533
+        temp_wav = tempfile.NamedTemporaryFile(dir=configuration.get_temporary_directory(), suffix='.wav').name
+        ffmpeg.convert_audio_to_wav(input_audio, temp_wav)
 
         logger.info('transcribing audio...')
 
         # Transcribe the audio using Whisper
         with warnings.catch_warnings():
-            result: WhisperResult = self.vad_model.transcribe(input_audio, vad=True, language=get_config().vad.language,
+            warnings.simplefilter("ignore")
+            result: WhisperResult = self.vad_model.transcribe(temp_wav, vad=True, language=get_config().vad.language,
                                                              temperature=0.0)
         voice_activity = []
 
-        logger.debug(result.to_dict())
+        logger.debug(json.dumps(result.to_dict(), indent=2))
 
         # Process the segments to extract tokens, timestamps, and confidence
         for i, segment in enumerate(result.segments):
@@ -196,6 +200,10 @@ class WhisperVADProcessor(VADProcessor):
                 else:
                     logger.info(
                         "Unknown single character segment, not skipping, but logging, please report if this is a mistake: " + segment.text)
+                    
+            if segment.no_speech_prob and segment.no_speech_prob > 0.6:
+                logger.debug(f"Skipping segment with high no_speech_prob: {segment.no_speech_prob} for segment {segment.text} at {segment.start}-{segment.end}")
+                continue
 
 
             logger.debug(segment.to_dict())
@@ -376,7 +384,7 @@ vad_processor = VADSystem()
 # Test cases for all VADProcessors
 def test_vad_processors():
     logger.setLevel(logging.DEBUG)
-    test_audio = r"C:\Users\Beangate\GSM\GameSentenceMiner\GameSentenceMiner\test\tmpso7v0265.opus"
+    test_audio = r"C:\Users\Beangate\GSM\GameSentenceMiner\GameSentenceMiner\test\NEKOPARAvol.1_2025-08-18-17-20-43-614.opus"
     output_dir = r"C:\Users\Beangate\GSM\GameSentenceMiner\GameSentenceMiner\test\output"
     os.makedirs(output_dir, exist_ok=True)
     processors = [
